import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { VorgangBescheidWizardE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-bescheid-wizard.e2e.component';
import { VorgangFormularButtonsE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-formular-buttons.e2e.components';
import { VorgangSubnavigationE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-subnavigation';
import { SmockerMocks } from 'apps/alfa-e2e/src/model/smocker';
import {
  EingangE2E,
  EingangHeaderE2E,
  VorgangE2E,
  VorgangStatusE2E,
} from 'apps/alfa-e2e/src/model/vorgang';
import 'cypress-real-events/support';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { addSmockerMock, dropCollections } from '../../../support/cypress-helper';
import { exist, haveText, notExist } from '../../../support/cypress.util';
import { initUsermanagerUsers, loginAsSabine } from '../../../support/user-util';
import {
  buildVorgang,
  createVorgang,
  initVorgaenge,
  objectIds,
} from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

//TODO: Jenkins konfigurieren
describe.skip('Upload automatic Bescheid', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const bescheidWizard: VorgangBescheidWizardE2EComponent = vorgangPage.getBescheidWizard();

  const formIdP: string = 'KFAS_STAGE_KI_10_Haltverbot_LANDESHACKATHON';
  const formEngineName: string = 'FormSolutions';
  const eingangHeader: EingangHeaderE2E = { ...createVorgang().eingangs[0].header };

  const eingangP: EingangE2E = {
    ...createVorgang().eingangs[0],
    header: { ...eingangHeader, formId: formIdP, formEngineName },
  };

  const bescheidAutomatik: VorgangE2E = {
    ...createVorgang(),
    status: VorgangStatusE2E.IN_BEARBEITUNG,
    name: 'automatischerBescheid',
    eingangs: [eingangP],
  };

  const standardVorgang: VorgangE2E = {
    ...buildVorgang(objectIds[1], 'Standard Vorgang'),
    status: VorgangStatusE2E.IN_BEARBEITUNG,
  };

  const vorgangFormularButtons: VorgangFormularButtonsE2EComponent =
    vorgangPage.getFormularButtons();

  const vorgangSubnavigation: VorgangSubnavigationE2EComponent = vorgangPage.getSubnavigation();

  before(() => {
    initVorgaenge([bescheidAutomatik, standardVorgang]);
    initUsermanagerUsers();

    addSmockerMock(SmockerMocks.SMARTDOCUMENT_MOCK);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe.skip('Upload automatic Bescheid document', () => {
    it('should show automatic Bescheid button', () => {
      vorgangList.getListItem(bescheidAutomatik.name).getRoot().click();
      waitForSpinnerToDisappear();

      vorgangFormularButtons.getBescheidenButton().click();
      bescheidWizard.getWeiterButton().click();
      exist(bescheidWizard.getUploadAutomaticBescheidButton());
    });

    it('should upload automatic Bescheid file', () => {
      waitForSpinnerToDisappear();
      bescheidWizard.getUploadAutomaticBescheidButton().click();

      exist(bescheidWizard.getBescheidUploadSpinner());
      notExist(bescheidWizard.getBescheidUploadSpinner());
      exist(bescheidWizard.getAutomaticBescheidFileInWizard());
    });

    it('should show automatic header and text in step 3', () => {
      bescheidWizard.getWeiterButton().click();
      waitForSpinnerToDisappear();

      haveText(bescheidWizard.getMailText(), '');
    });
  });

  describe('Do not show automatic upload button on other Vorgang', () => {
    it('should not show button on Vorgang with different formID', () => {
      bescheidWizard.getCloseButton().click();
      bescheidWizard.getCloseVerwerfenButton().click();
      vorgangSubnavigation.getBackButton().click();
      vorgangList.getListItem(standardVorgang.name).getRoot().click();
      waitForSpinnerToDisappear();

      vorgangFormularButtons.getBescheidenButton().click();
      bescheidWizard.getWeiterButton().click();

      notExist(bescheidWizard.getUploadAutomaticBescheidButton());
    });
  });
});
