import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { VorgangZusammenarbeitE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-zusammenarbeit.e2e.component';
import { VorgangE2E, VorgangStatusE2E } from 'apps/alfa-e2e/src/model/vorgang';
import 'cypress-real-events/support';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections } from '../../../support/cypress-helper';
import { exist, notExist } from '../../../support/cypress.util';
import { initUsermanagerUsers, loginAsSabine } from '../../../support/user-util';
import { buildVorgang, initVorgaenge, objectIds } from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang Zusammenarbeit anfragen', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const zusammenarbeitContainer: VorgangZusammenarbeitE2EComponent =
    vorgangPage.getZusammenarbeitContainer();

  const zusammenarbeitVorgang: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'Zusammenarbeit Vorgang'),
    status: VorgangStatusE2E.IN_BEARBEITUNG,
  };

  before(() => {
    initVorgaenge([zusammenarbeitVorgang]);
    initUsermanagerUsers();

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('create new Anfrage', () => {
    it('should show button for Zusammenarbeit', () => {
      vorgangList.getListItem(zusammenarbeitVorgang.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(zusammenarbeitContainer.getAnfrageButton());
    });

    it('should show input elements on Anfrage click', () => {
      zusammenarbeitContainer.createAnfrage();

      exist(zusammenarbeitContainer.getZustaendigeStelleButton());
      exist(zusammenarbeitContainer.getStelleTitel());
      exist(zusammenarbeitContainer.getStelleNachricht());
      exist(zusammenarbeitContainer.getSendButton());
      exist(zusammenarbeitContainer.getCancelButton());
    });

    it('should close elements on Cancel click', () => {
      zusammenarbeitContainer.cancelAnfrage();

      notExist(zusammenarbeitContainer.getZustaendigeStelleButton());
      notExist(zusammenarbeitContainer.getStelleTitel());
      notExist(zusammenarbeitContainer.getStelleNachricht());
      notExist(zusammenarbeitContainer.getSendButton());
      notExist(zusammenarbeitContainer.getCancelButton());
      exist(zusammenarbeitContainer.getAnfrageButton());
    });

    it('should open new search label for Zustaendige Stelle', () => {
      //button click
      //Layer wird angezeigt
    });

    it('should close layer on Cancel click', () => {
      //click Abbrechen
    });

    it('should show no search on entering 1 element', () => {
      //1 Zeichen in Suche eingeben
      //keine Vorschau
    });

    it('should show results on entering 2 elements', () => {
      //2 Zeichen in Suche eingeben
      //Vorschau kontrollieren
    });

    it('should delete search term on clicking X', () => {
      //X in Suche klicken
      //Suche ist leer
      //keine Vorschau
    });

    it('should copy and paste Zustaendige Stelle after selection', () => {
      //click Suchergebnis
      //Adresse und Name wird übernommen
      //Layer ist geschlossen
    });

    it('should show title and message read-only and remove buttons after sending', () => {
      //Titel eingeben
      //Nachricht eingeben
      //Button klicken
      //Titel und Datum werden angezeigt
      //Nachricht wird angezeigt
      //Buttons werden ausgeblendet
    });
  });
});
