import { ConvertForDataTestPipe } from '@alfa-client/tech-shared';
import { getElementFromFixture, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormControl, NgControl, ReactiveFormsModule, ValidationErrors } from '@angular/forms';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { TextFieldComponent } from './text-field.component';

describe('TextFieldComponent', () => {
  let component: TextFieldComponent;
  let fixture: ComponentFixture<TextFieldComponent>;

  const label = 'custom';
  const spanSelector = getDataTestIdOf('text-field-span-' + label);
  const inputSelector = getDataTestIdOf('text-field-input-' + label);
  const errorsSelector = getDataTestIdOf('text-field-errors-' + label);

  const formControl = new FormControl('');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [TextFieldComponent, ConvertForDataTestPipe],
      imports: [ReactiveFormsModule],
    })
      .overrideComponent(TextFieldComponent, {
        add: {
          providers: [
            {
              provide: NgControl,
              useValue: formControl,
            },
          ],
        },
      })
      .compileComponents();

    fixture = TestBed.createComponent(TextFieldComponent);
    component = fixture.componentInstance;
    component.label = label;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should use label', () => {
    const labelElement = getElementFromFixture(fixture, spanSelector);
    expect(labelElement.textContent).toBe(label);
  });

  it('should use form control', () => {
    const fieldText = 'text';
    component.writeValue(fieldText);

    fixture.detectChanges();

    const inputElement = getElementFromFixture(fixture, inputSelector);
    expect(inputElement.value).toBe(fieldText);
  });

  describe('invalid indication', () => {
    it('should show as red if invalid', () => {
      formControl.setErrors({ someErrorCode: 'Invalid' });

      fixture.detectChanges();

      const labelElement = getElementFromFixture(fixture, spanSelector);
      expect([...labelElement.classList]).toEqual(['text-red-500', 'font-bold']);
    });

    it('should not show as red if valid', () => {
      formControl.setErrors(null);

      fixture.detectChanges();

      const labelElement = getElementFromFixture(fixture, spanSelector);
      expect([...labelElement.classList]).toEqual([]);
    });
  });

  describe('error messages', () => {
    it('should not show empty error message container', () => {
      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, errorsSelector);
    });

    it('should show error messages', () => {
      const errors: ValidationErrors = {
        firstMessage: 'first',
        secondMessage: 'second',
      };
      formControl.setErrors(errors);

      fixture.detectChanges();

      const errorsElement: HTMLElement = getElementFromFixture(fixture, errorsSelector);
      const spansTexts = Array.from(errorsElement.querySelectorAll('span')).map(
        (span) => span.textContent,
      );
      expect(spansTexts).toEqual(Object.values(errors));
    });
  });
});
