import { TestBed } from '@angular/core/testing';

import { Mock, mock } from '@alfa-client/test-utils';
import { Dialog, DialogConfig } from '@angular/cdk/dialog';
import { OzgcloudDialogService } from './ozgcloud-dialog.service';

describe('OzgcloudDialogService', () => {
  let service: OzgcloudDialogService;
  let dialog: Mock<Dialog>;

  const component = <any>{ name: 'Component' };
  const dialogData = { id: 'ZumBeispiel' };
  const dialogConfigWithData: DialogConfig = { data: dialogData };
  const viewContainerRef = <any>{};
  const dialogConfigWithDataAndViewContainerRef: DialogConfig = {
    data: dialogData,
    viewContainerRef,
  };
  const dialogConfigWithOutDataAndWithViewContainerRef: DialogConfig = {
    viewContainerRef,
  };

  beforeEach(() => {
    dialog = mock(Dialog);

    TestBed.configureTestingModule({
      providers: [
        {
          provide: Dialog,
          useValue: dialog,
        },
      ],
    });
    service = TestBed.inject(OzgcloudDialogService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  describe('open', () => {
    it('should open dialog with data', () => {
      service.open(component, dialogData);

      expect(dialog.open).toHaveBeenCalledWith(component, dialogConfigWithData);
    });

    it('should open dialog wihtout data', () => {
      service.open(component);

      expect(dialog.open).toHaveBeenCalledWith(component, undefined);
    });
  });

  describe('openInCallingComponentContext', () => {
    it('should open dialog with data', () => {
      service.openInCallingComponentContext(component, viewContainerRef, dialogData);

      expect(dialog.open).toHaveBeenCalledWith(component, dialogConfigWithDataAndViewContainerRef);
    });

    it('should open dialog wihtout data', () => {
      service.openInCallingComponentContext(component, viewContainerRef);

      expect(dialog.open).toHaveBeenCalledWith(
        component,
        dialogConfigWithOutDataAndWithViewContainerRef,
      );
    });
  });

  describe('openWizard', () => {
    it('should open wizard dialog', () => {
      service.openWizard(component);

      expect(dialog.open).toHaveBeenCalledWith(component, {
        ...service.WIZARD_DIALOG_CONFIG,
      });
    });

    it('should open wizard dialog with data', () => {
      service.openWizard(component, dialogData);

      expect(dialog.open).toHaveBeenCalledWith(component, {
        ...service.WIZARD_DIALOG_CONFIG,
        data: dialogData,
      });
    });
  });
});
