import { BescheidLinkRel, BescheidResource, BescheidService } from '@alfa-client/bescheid-shared';
import { CommandOrder, CommandResource } from '@alfa-client/command-shared';
import {
  createEmptyStateResource,
  createStateResource,
  StateResource,
} from '@alfa-client/tech-shared';
import {
  dispatchEventFromFixture,
  existsAsHtmlElement,
  getElementFromFixture,
  Mock,
  mock,
  notExistsAsHtmlElement,
} from '@alfa-client/test-utils';
import { OzgcloudButtonWithSpinnerComponent } from '@alfa-client/ui';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ButtonWithSpinnerComponent } from '@ods/component';
import {
  createBescheidResource,
  createBescheidStateResource,
} from 'libs/bescheid-shared/src/test/bescheid';
import { CommandLinkRel } from 'libs/command-shared/src/lib/command.linkrel';
import {
  createCommandResource,
  createCommandStateResource,
} from 'libs/command-shared/test/command';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { BehaviorSubject, EMPTY, first, of } from 'rxjs';
import { singleColdCompleted } from '../../../../../../tech-shared/test/marbles';
import { BescheidenFormService } from '../bescheiden.formservice';
import { VorgangDetailBescheidenResultAttachmentsComponent } from './vorgang-detail-bescheiden-result-attachments/vorgang-detail-bescheiden-result-attachments.component';
import { VorgangDetailBescheidenResultDokumentComponent } from './vorgang-detail-bescheiden-result-dokument/vorgang-detail-bescheiden-result-dokument.component';
import { VorgangDetailBescheidenResultNachrichtComponent } from './vorgang-detail-bescheiden-result-nachricht/vorgang-detail-bescheiden-result-nachricht.component';
import { VorgangDetailBescheidenResultStatusComponent } from './vorgang-detail-bescheiden-result-status/vorgang-detail-bescheiden-result-status.component';
import { VorgangDetailBescheidenResultComponent } from './vorgang-detail-bescheiden-result.component';

describe('VorgangDetailBescheidenResultComponent', () => {
  let component: VorgangDetailBescheidenResultComponent;
  let fixture: ComponentFixture<VorgangDetailBescheidenResultComponent>;

  let bescheidService: Mock<BescheidService>;
  let formService: Mock<BescheidenFormService>;

  const bescheidStatus: string = getDataTestIdOf('bescheid-status');
  const bescheidDocument: string = getDataTestIdOf('bescheid-document');
  const bescheidAttachments: string = getDataTestIdOf('bescheid-attachments');
  const saveAndSendButton: string = getDataTestIdOf('confirm-and-save-button');
  const sendButton: string = getDataTestIdOf('send-button');
  const nachrichtAntragstellerComponent = getDataTestIdOf('bescheid-nachricht-an-antragsteller');

  beforeEach(async () => {
    bescheidService = mock(BescheidService);
    bescheidService.getBescheidDraft.mockReturnValue(EMPTY);

    formService = mock(BescheidenFormService);
    formService.getBescheidChanges.mockReturnValue(
      new BehaviorSubject({ beschiedenAm: new Date(), bewilligt: false }),
    );
    formService.getActiveStep.mockReturnValue(EMPTY);
    formService.isSendByManual.mockReturnValue(EMPTY);

    await TestBed.configureTestingModule({
      declarations: [
        VorgangDetailBescheidenResultComponent,
        MockComponent(VorgangDetailBescheidenResultStatusComponent),
        MockComponent(VorgangDetailBescheidenResultDokumentComponent),
        MockComponent(VorgangDetailBescheidenResultAttachmentsComponent),
        MockComponent(OzgcloudButtonWithSpinnerComponent),
        MockComponent(VorgangDetailBescheidenResultNachrichtComponent),
        MockComponent(ButtonWithSpinnerComponent),
      ],
      providers: [
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
        {
          provide: BescheidenFormService,
          useValue: formService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangDetailBescheidenResultComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('render Nachricht component', () => {
    it('should render', () => {
      formService.getActiveStep.mockReturnValue(of(3));
      formService.isSendByManual.mockReturnValue(of(false));
      component.ngOnInit();
      component.canSend$ = of(true);

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, nachrichtAntragstellerComponent);
      expect(element).toBeInstanceOf(HTMLElement);
    });

    it.each([1, 2])('should not render in step %d', (step) => {
      formService.getActiveStep.mockReturnValue(of(step));
      formService.isSendByManual.mockReturnValue(of(false));
      component.ngOnInit();
      component.canSend$ = of(true);

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, nachrichtAntragstellerComponent);
      expect(element).not.toBeInstanceOf(HTMLElement);
    });

    it('should not render if send by manual', () => {
      formService.getActiveStep.mockReturnValue(of(3));
      formService.isSendByManual.mockReturnValue(of(true));
      component.ngOnInit();
      component.canSend$ = of(true);

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, nachrichtAntragstellerComponent);
      expect(element).not.toBeInstanceOf(HTMLElement);
    });

    it('should not render if postfach not configured', () => {
      formService.getActiveStep.mockReturnValue(of(3));
      formService.isSendByManual.mockReturnValue(of(false));
      component.ngOnInit();
      component.canSend$ = of(false);

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, nachrichtAntragstellerComponent);
      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('render send button', () => {
    it('should render', () => {
      formService.getActiveStep.mockReturnValue(of(3));
      formService.isSendByManual.mockReturnValue(of(false));
      bescheidService.getBescheidDraft.mockReturnValue(
        of(createStateResource(createBescheidResource())),
      );
      component.ngOnInit();
      component.canSend$ = of(true);

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, sendButton);
      expect(element).toBeInstanceOf(HTMLElement);
    });

    it.each([1, 2])('should not render in step %d', (step) => {
      formService.getActiveStep.mockReturnValue(of(step));
      formService.isSendByManual.mockReturnValue(of(false));
      component.ngOnInit();
      component.canSend$ = of(true);

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, sendButton);
      expect(element).not.toBeInstanceOf(HTMLElement);
    });

    it('should not render if send by manual', () => {
      formService.getActiveStep.mockReturnValue(of(3));
      formService.isSendByManual.mockReturnValue(of(true));
      component.ngOnInit();
      component.canSend$ = of(true);

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, sendButton);
      expect(element).not.toBeInstanceOf(HTMLElement);
    });

    it('should not render if postfach not configured', () => {
      formService.getActiveStep.mockReturnValue(of(3));
      formService.isSendByManual.mockReturnValue(of(false));
      component.ngOnInit();
      component.canSend$ = of(false);

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, sendButton);
      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('ngOnInit', () => {
    it('should call service to get bescheid document file', () => {
      component.ngOnInit();

      expect(bescheidService.getBescheidDocumentFile).toHaveBeenCalled();
    });

    it('should call service to get bescheid draft', () => {
      component.ngOnInit();

      expect(bescheidService.getBescheidDraft).toHaveBeenCalled();
    });

    it('should call service to get bescheid document', () => {
      component.ngOnInit();

      expect(bescheidService.getBescheidDocument).toHaveBeenCalled();
    });

    it('should call formservice to get current bescheid/formular', () => {
      component.ngOnInit();

      expect(formService.getBescheidChanges).toHaveBeenCalled();
    });

    it('should get is send by manual', () => {
      component.getIsSendManually = jest.fn();

      component.ngOnInit();

      expect(component.getIsSendManually).toHaveBeenCalled();
    });

    it('should call service to get upload bescheid document in progress', () => {
      component.ngOnInit();

      expect(bescheidService.getUploadBescheidDocumentInProgress).toHaveBeenCalled();
    });

    it('should call service to get create bescheid document in progress', () => {
      component.ngOnInit();

      expect(bescheidService.getCreateBescheidDocumentInProgress).toHaveBeenCalled();
    });

    it('should get active step', () => {
      component.getActiveStep = jest.fn();

      component.ngOnInit();

      expect(component.getActiveStep).toHaveBeenCalled();
    });

    describe('canSave$', () => {
      it('should emit true', () => {
        bescheidService.getBescheidDraft.mockReturnValue(
          of(createBescheidStateResource([BescheidLinkRel.BESCHEIDEN])),
        );
        component.ngOnInit();

        expect(component.canSave$).toBeObservable(singleColdCompleted(true));
      });

      it('should emit false', () => {
        bescheidService.getBescheidDraft.mockReturnValue(of(createBescheidStateResource()));
        component.ngOnInit();

        expect(component.canSave$).toBeObservable(singleColdCompleted(false));
      });
    });

    describe('canSend$', () => {
      it('should emit true', () => {
        bescheidService.getBescheidDraft.mockReturnValue(
          of(createBescheidStateResource([BescheidLinkRel.BESCHEIDEN_UND_SENDEN])),
        );
        component.ngOnInit();

        expect(component.canSend$).toBeObservable(singleColdCompleted(true));
      });

      it('should emit false', () => {
        bescheidService.getBescheidDraft.mockReturnValue(of(createBescheidStateResource()));
        component.ngOnInit();

        expect(component.canSend$).toBeObservable(singleColdCompleted(false));
      });
    });

    describe('showMissingBescheidDocumentError$', () => {
      it('should emit true', () => {
        formService.getShowMissingBescheidDocumentError.mockReturnValue(of(true));

        component.ngOnInit();

        expect(component.showMissingBescheidDocumentError$).toBeObservable(
          singleColdCompleted(true),
        );
      });

      it('should emit false', () => {
        formService.getShowMissingBescheidDocumentError.mockReturnValue(of(false));

        component.ngOnInit();

        expect(component.showMissingBescheidDocumentError$).toBeObservable(
          singleColdCompleted(false),
        );
      });
    });
  });

  describe('deleteBescheidDocument', () => {
    it('should call service to delete bescheid document', () => {
      component.deleteBescheidDocument();

      expect(bescheidService.deleteBescheidDocument).toHaveBeenCalled();
    });

    it('should clear bescheid document file', () => {
      component.deleteBescheidDocument();

      expect(formService.clearBescheidDocumentFile).toHaveBeenCalled();
    });
  });

  describe('save and send manually', () => {
    const bescheidDraft: BescheidResource = createBescheidResource();
    const bescheidStateResource: StateResource<BescheidResource> =
      createStateResource(bescheidDraft);

    beforeEach(() => {
      component.bescheidDraftStateResource$ = of(bescheidStateResource);
      component.sendByManual$ = of(true);
      component.activeStep$ = of(3);
    });

    it('should clear nachricht', () => {
      component.doUpdateAndSend = jest.fn();

      component.saveAndSendManually(bescheidDraft);

      expect(formService.clearNachricht).toHaveBeenCalled();
    });

    it('should call component on event dispatch', () => {
      component.saveAndSendManually = jest.fn();
      component.canSave$ = of(true);

      fixture.detectChanges();

      dispatchEventFromFixture(fixture, saveAndSendButton, 'clickEmitter');

      expect(component.saveAndSendManually).toHaveBeenCalledWith(bescheidDraft);
    });

    it('should call do update and send', () => {
      component.doUpdateAndSend = jest.fn();

      component.saveAndSendManually(bescheidDraft);

      expect(component.doUpdateAndSend).toHaveBeenCalled();
    });
  });

  describe('save and send with Nachricht', () => {
    const bescheidDraft: BescheidResource = createBescheidResource();
    const bescheidStateResource: StateResource<BescheidResource> =
      createStateResource(bescheidDraft);
    let doUpdateAndSendMock: jest.Mock;

    beforeEach(() => {
      component.bescheidDraftStateResource$ = of(bescheidStateResource);
      component.sendByManual$ = of(true);
      component.activeStep$ = of(3);
      doUpdateAndSendMock = component.doUpdateAndSend = jest.fn();
    });

    it('should call do update and send', () => {
      component.saveAndSendWithNachricht(bescheidDraft);

      expect(doUpdateAndSendMock).toHaveBeenCalledWith(bescheidDraft, expect.any(Function));
    });

    it('should send bescheid to antragsteller', () => {
      bescheidService.sendBescheidToAntragsteller.mockReturnValue(of(createCommandStateResource()));
      component.saveAndSendWithNachricht(bescheidDraft);
      const sendCallback = doUpdateAndSendMock.mock.calls[0][1];

      sendCallback(bescheidDraft);

      expect(bescheidService.sendBescheidToAntragsteller).toHaveBeenCalledWith(bescheidDraft);
    });

    it('should emit send nachricht', () => {
      const sendCommandStateResource: StateResource<CommandResource> = createCommandStateResource();
      bescheidService.sendBescheidToAntragsteller.mockReturnValue(of(sendCommandStateResource));
      component.saveAndSendWithNachricht(bescheidDraft);
      const sendCallback = doUpdateAndSendMock.mock.calls[0][1];

      sendCallback(bescheidDraft).subscribe();

      expect(component.sendWithNachricht$.value).toEqual(sendCommandStateResource);
    });
  });

  describe('doUpdateAndSend', () => {
    const bescheidDraft: BescheidResource = createBescheidResource();
    const commandStateResource: StateResource<CommandResource> = createStateResource(
      createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
    );

    beforeEach(() => {
      component.sendByManual$ = of(true);
      formService.submit = jest.fn().mockReturnValue(of(commandStateResource));
    });

    it('should call formservice submit', () => {
      component.doUpdateAndSend(bescheidDraft, jest.fn()).pipe(first()).subscribe();

      expect(formService.submit).toHaveBeenCalled();
    });

    it('should call send after successfully update', () => {
      component.doSendAfterSuccessfullyUpdate = jest.fn();
      const sendMock = jest.fn();

      component.doUpdateAndSend(bescheidDraft, sendMock).subscribe();

      expect(component.doSendAfterSuccessfullyUpdate).toHaveBeenCalledWith(bescheidDraft, sendMock);
    });

    it('should close dialog on successfully done command', () => {
      const sendMock = jest.fn();
      formService.submit = jest.fn().mockReturnValue(
        of(
          createStateResource({
            ...createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
            order: CommandOrder.SEND_BESCHEID,
          }),
        ),
      );
      const emit = (component.closeDialog.emit = jest.fn());

      component.doUpdateAndSend(bescheidDraft, sendMock).subscribe();

      expect(emit).toHaveBeenCalled();
    });
  });

  describe('doSendAfterSuccessfullyUpdate', () => {
    const bescheidDraft: BescheidResource = createBescheidResource();

    it('should call send', () => {
      const commandStateResource: StateResource<CommandResource> = createStateResource(
        createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
      );
      const sendMock = jest.fn();

      component
        .doSendAfterSuccessfullyUpdate(bescheidDraft, sendMock)(of(commandStateResource))
        .subscribe();

      expect(sendMock).toHaveBeenCalled();
    });

    it('should not call send', () => {
      const commandStateResource: StateResource<CommandResource> = createEmptyStateResource();
      const sendMock = jest.fn();

      component
        .doSendAfterSuccessfullyUpdate(bescheidDraft, sendMock)(of(commandStateResource))
        .subscribe();

      expect(sendMock).not.toHaveBeenCalled();
    });
  });

  describe('bescheid status text', () => {
    it('should NOT be visible on NOT send by manual', () => {
      component.sendByManual$ = of(false);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidStatus);
    });

    it('should NOT be visible on step 3 and NOT send by manual', () => {
      component.sendByManual$ = of(false);
      component.activeStep$ = of(3);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidStatus);
    });
  });

  describe('bescheid document', () => {
    it('should NOT be visible on NOT send by manual', () => {
      component.sendByManual$ = of(false);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidDocument);
    });

    it('should NOT be visible on step 3', () => {
      component.sendByManual$ = of(false);
      component.activeStep$ = of(3);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidDocument);
    });

    it('should NOT be visible on step 1', () => {
      component.sendByManual$ = of(false);
      component.activeStep$ = of(1);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidDocument);
    });

    it('should be visible in step 3 when link exists', () => {
      component.sendByManual$ = of(false);
      component.activeStep$ = of(3);
      component.canSend$ = of(true);

      fixture.detectChanges();

      existsAsHtmlElement(fixture, bescheidDocument);
    });
  });

  describe('bescheid attachments', () => {
    it('should NOT be visible on NOT send by manual', () => {
      component.sendByManual$ = of(false);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidAttachments);
    });

    it('should NOT be visible on step 3', () => {
      component.sendByManual$ = of(false);
      component.activeStep$ = of(3);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidAttachments);
    });

    it('should NOT be visible on step 1', () => {
      component.sendByManual$ = of(false);
      component.activeStep$ = of(1);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidAttachments);
    });

    it('should be visible in step 3 when postfach is configured', () => {
      component.sendByManual$ = of(false);
      component.activeStep$ = of(3);
      component.canSend$ = of(true);

      fixture.detectChanges();

      existsAsHtmlElement(fixture, bescheidAttachments);
    });
  });

  describe('update and bescheiden', () => {
    it('should NOT be visible on step 1', () => {
      component.sendByManual$ = of(false);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, saveAndSendButton);
    });

    it('should NOT be visible on setp 2', () => {
      component.sendByManual$ = of(false);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, saveAndSendButton);
    });
    it('should NOT be visible on step 3 and NOT send by manual', () => {
      component.sendByManual$ = of(false);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, saveAndSendButton);
    });
  });

  describe('resetSend', () => {
    it('should set to empty state resource', () => {
      component.saveAndSendInProgress$ = of(createStateResource(createCommandResource()));

      component.resetSend();

      expect(component.saveAndSendInProgress$).toBeObservable(
        singleColdCompleted(createEmptyStateResource()),
      );
    });
  });

  describe('get active step', () => {
    it('should call formService', () => {
      component.getActiveStep().pipe(first()).subscribe();

      expect(formService.getActiveStep).toHaveBeenCalled();
    });

    it.each([1, VorgangDetailBescheidenResultComponent.BESCHEID_VERSENDEN_STEP])(
      'should call bescheidService to clear attachment upload on step %s',
      (step: number) => {
        formService.getActiveStep.mockReturnValue(of(step));

        component.getActiveStep().pipe(first()).subscribe();

        expect(bescheidService.clearAttachmentUpload).toHaveBeenCalled();
      },
    );

    describe('reset send', () => {
      beforeEach(() => {
        component.resetSend = jest.fn();
      });

      it.each([1, VorgangDetailBescheidenResultComponent.ADD_DOCUMENTS_STEP])(
        'should be called on step %s',
        (step: number) => {
          formService.getActiveStep.mockReturnValue(of(step));

          component.getActiveStep().pipe(first()).subscribe();

          expect(component.resetSend).toHaveBeenCalled();
        },
      );

      it('should not be called on last step', () => {
        formService.getActiveStep.mockReturnValue(of(3));

        component.getActiveStep().pipe(first()).subscribe();

        expect(component.resetSend).not.toHaveBeenCalled();
      });
    });
  });

  describe('reset send', () => {
    it('should clear saveAndSendInProgress', (done) => {
      component.saveAndSendInProgress$ = of(createCommandStateResource());

      component.resetSend();

      component.saveAndSendInProgress$
        .pipe(first())
        .subscribe((saveAndSendInProgress: StateResource<CommandResource>) => {
          expect(saveAndSendInProgress).toEqual(createEmptyStateResource());
          done();
        });
    });

    it('should clear sendWithNachricht', () => {
      component.sendWithNachricht$.next(createCommandStateResource());

      component.resetSend();

      expect(component.sendWithNachricht$.value).toEqual(createEmptyStateResource());
    });
  });

  describe('getIsSendManually', () => {
    beforeEach(() => {
      component.resetSend = jest.fn();
    });

    it('should get is send by manual from from service', () => {
      component.getIsSendManually();

      expect(formService.isSendByManual).toHaveBeenCalled();
    });

    it.each([true, false])(
      'should reset send if send by manual is %s',
      (isSendByManual: boolean) => {
        formService.isSendByManual.mockReturnValue(of(isSendByManual));

        component.getIsSendManually().subscribe();

        expect(component.resetSend).toHaveBeenCalled();
      },
    );
  });
});
