/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.command;

import lombok.AccessLevel;
import lombok.Getter;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor(access = AccessLevel.PRIVATE)
@Getter
public enum CommandOrder {

	VORGANG_ANNEHMEN(true, Type.VORGANG),
	VORGANG_VERWERFEN(true, Type.VORGANG),
	VORGANG_ZURUECKHOLEN(true, Type.VORGANG),
	VORGANG_BEARBEITEN(true, Type.VORGANG),
	VORGANG_BESCHEIDEN(true, Type.VORGANG),
	VORGANG_ZURUECKSTELLEN(true, Type.VORGANG),
	VORGANG_ABSCHLIESSEN(true, Type.VORGANG),
	VORGANG_WIEDEREROEFFNEN(true, Type.VORGANG),

	VORGANG_ZUM_LOESCHEN_MARKIEREN(true, Type.VORGANG),
	VORGANG_LOESCHEN(false, Type.VORGANG_LIST),
	LOESCH_ANFORDERUNG_ZURUECKNEHMEN(true, Type.VORGANG),

	ASSIGN_USER(false, Type.VORGANG),

	SET_AKTENZEICHEN(false, Type.VORGANG),

	REDIRECT_VORGANG(false, Type.FORWARDING),
	FORWARD_SUCCESSFULL(false, Type.FORWARDING),
	FORWARD_FAILED(false, Type.FORWARDING),

	@Deprecated
	SEND_POSTFACH_MAIL(false, Type.POSTFACH),
	SEND_POSTFACH_NACHRICHT(false, Type.POSTFACH),
	RESEND_POSTFACH_MAIL(false, Type.POSTFACH),
	RECEIVE_POSTFACH_NACHRICHT(false, Type.POSTFACH),

	CREATE_ATTACHED_ITEM(false, Type.VORGANG),
	UPDATE_ATTACHED_ITEM(false, Type.VORGANG),
	PATCH_ATTACHED_ITEM(false, Type.VORGANG),
	DELETE_ATTACHED_ITEM(false, Type.VORGANG),

	CREATE_BESCHEID(false, Type.BESCHEID),
	DELETE_BESCHEID(false, Type.VORGANG),
	UPDATE_BESCHEID(false, Type.BESCHEID),

	CREATE_BESCHEID_DOCUMENT(false, Type.DOCUMENT),
	CREATE_BESCHEID_DOCUMENT_FROM_FILE(false, Type.DOCUMENT),
	SEND_BESCHEID(false, Type.BESCHEID),

	PROCESS_VORGANG(false, Type.VORGANG),

	UNBEKANNT(false, Type.NONE);

	enum Type {
		VORGANG, VORGANG_LIST, WIEDERVORLAGE, KOMMENTAR, FORWARDING, POSTFACH, BESCHEID, DOCUMENT, NONE
	}

	private final boolean revokeable;
	@Getter(value = AccessLevel.PROTECTED)
	private final Type type;

	public static CommandOrder fromOrder(String order) {
		try {
			return valueOf(order);
		} catch (IllegalArgumentException e) {
			return UNBEKANNT;
		}
	}

}